import React, { useEffect, useState } from "react";
import { Login } from "./components/Login";
import { Register } from "./components/Register";
import { MainPage } from "./components/MainPage";
import { ThemeToggle } from "./components/ThemeToggle";

const TOKEN_STORAGE_KEY = "litecloud_token";
const THEME_STORAGE_KEY = "litecloud_theme";

export const App: React.FC = () => {
  const [token, setToken] = useState<string | null>(null);
  const [loading, setLoading] = useState(true);
  const [showRegister, setShowRegister] = useState(false);
  const [isDarkMode, setIsDarkMode] = useState(true);

  useEffect(() => {
    const savedToken = localStorage.getItem(TOKEN_STORAGE_KEY);
    if (savedToken) {
      setToken(savedToken);
    }

    const savedTheme = localStorage.getItem(THEME_STORAGE_KEY);
    if (savedTheme !== null) {
      setIsDarkMode(savedTheme === 'dark');
    }

    setLoading(false);
  }, []);

  const handleLoginSuccess = (token: string) => {
    localStorage.setItem(TOKEN_STORAGE_KEY, token);
    setToken(token);
  };

  const handleLogout = () => {
    localStorage.removeItem(TOKEN_STORAGE_KEY);
    setToken(null);
    setShowRegister(false);
  };

  const toggleTheme = () => {
    const newTheme = !isDarkMode;
    setIsDarkMode(newTheme);
    localStorage.setItem(THEME_STORAGE_KEY, newTheme ? 'dark' : 'light');
  };

  if (loading) {
    return (
      <div className="loading-container">
        <div className="loading-spinner"></div>
        <p>Loading...</p>
      </div>
    );
  }

  if (!token) {
    return (
      <>
        <ThemeToggle isDarkMode={isDarkMode} onToggle={toggleTheme} />
        {showRegister ? (
          <Register 
            onSwitchToLogin={() => setShowRegister(false)} 
            isDarkMode={isDarkMode}
          />
        ) : (
          <Login
            onLoginSuccess={handleLoginSuccess}
            onSwitchToRegister={() => setShowRegister(true)}
            isDarkMode={isDarkMode}
          />
        )}
      </>
    );
  }

  return (
    <>
      <ThemeToggle isDarkMode={isDarkMode} onToggle={toggleTheme} />
      <MainPage isDarkMode={isDarkMode} onLogout={handleLogout} />
    </>
  );
};