import React, { useState, useEffect, useRef } from "react";
import { ChevronDown, Upload, Share, Trash2 } from "lucide-react";

interface File {
  id: number;
  user_id: number;
  original_name: string;
  storage_path: string;
  uploaded_at: string;
  size: number;
}

interface FilesComponentProps {
  isDarkMode: boolean;
}

export const FilesComponent: React.FC<FilesComponentProps> = ({ isDarkMode }) => {
  const [files, setFiles] = useState<File[]>([]);
  const [loading, setLoading] = useState(false);
  const [error, setError] = useState<string | null>(null);
  const [success, setSuccess] = useState<string | null>(null);
  const [openDropdown, setOpenDropdown] = useState<number | null>(null);
  const [token, setToken] = useState<string | null>(null);

  const fileInputRef = useRef<HTMLInputElement>(null);
  const dropdownRef = useRef<HTMLTableDataCellElement | null>(null);

  // Recupera il token solo una volta al mount o quando cambia
  useEffect(() => {
    const storedToken = localStorage.getItem('token');
    setToken(storedToken);
  }, []);

  const apiCall = async (url: string, options: RequestInit = {}) => {
    if (!token) throw new Error('No auth token found');
    const response = await fetch(url, {
      ...options,
      headers: {
        'Authorization': `Bearer ${token}`,
        'Content-Type': 'application/json',
        ...options.headers,
      },
    });

    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }

    return response;
  };

  const fetchFiles = async () => {
    if (!token) return; // Non chiamare se il token non è pronto
    try {
      setLoading(true);
      setError(null);
      const response = await apiCall('http://192.168.1.120:8082/api/files');
      const filesData = await response.json();
      setFiles(filesData);
    } catch (err) {
      setError('Failed to fetch files');
    } finally {
      setLoading(false);
    }
  };

  useEffect(() => {
    if (token) fetchFiles();
  }, [token]);

  useEffect(() => {
    const handleClickOutside = (event: MouseEvent) => {
      if (dropdownRef.current && !dropdownRef.current.contains(event.target as Node)) {
        setOpenDropdown(null);
      }
    };

    document.addEventListener('mousedown', handleClickOutside);
    return () => document.removeEventListener('mousedown', handleClickOutside);
  }, []);

  const handleFileUpload = async (event: React.ChangeEvent<HTMLInputElement>) => {
    const file = event.target.files?.[0];
    if (!file) return;

    try {
      setLoading(true);
      setError(null);
      const formData = new FormData();
      formData.append('file', file);
      formData.append('filename', file.name);

      const token = getAuthToken();
      const response = await fetch('http://192.168.1.120:8082/api/upload', {
        method: 'POST',
        headers: {
          'Authorization': `Bearer ${token}`,
        },
        body: formData,
      });

      if (response.ok) {
        setSuccess('File uploaded successfully!');
        fetchFiles();
      } else {
        throw new Error('Upload failed');
      }
    } catch (err) {
      setError('Failed to upload file');
    } finally {
      setLoading(false);
      if (fileInputRef.current) {
        fileInputRef.current.value = '';
      }
    }
  };

  const handleFileAction = async (action: 'share' | 'delete', fileId: number) => {
    try {
      setLoading(true);
      setError(null);
      
      if (action === 'share') {
        const response = await apiCall('http://192.168.1.120:8082/api/share', {
          method: 'POST',
          body: JSON.stringify({
            file_id: fileId,
            expires_in_days: 7
          }),
        });
        const shareId = await response.json();
        setSuccess(`Share link created: /shared/${shareId}`);
        
      } else if (action === 'delete') {
        await apiCall(`http://192.168.1.120:8082/api/files/${fileId}`, {
          method: 'DELETE',
        });
        setSuccess('File deleted successfully');
        fetchFiles();
      }
    } catch (err) {
      setError(`Failed to ${action} file`);
    } finally {
      setLoading(false);
      setOpenDropdown(null);
    }
  };

  const formatDate = (dateString: string) => {
    return new Date(dateString).toLocaleString();
  };

  const formatFileSize = (bytes: number) => {
    const sizes = ['B', 'KB', 'MB', 'GB'];
    if (bytes === 0) return '0 B';
    const i = Math.floor(Math.log(bytes) / Math.log(1024));
    return Math.round(bytes / Math.pow(1024, i) * 100) / 100 + ' ' + sizes[i];
  };

  return (
    <div className="component-container">
      {/* Success/Error Messages */}
      {error && (
        <div className="notification notification-error">
          {error}
          <button onClick={() => setError(null)} className="notification-close">×</button>
        </div>
      )}
      {success && (
        <div className="notification notification-success">
          {success}
          <button onClick={() => setSuccess(null)} className="notification-close">×</button>
        </div>
      )}

      <div className="component-header">
        <h1 className={`component-title ${isDarkMode ? 'text-white' : 'text-gray-800'}`}>
          My Files
        </h1>
        <input
          ref={fileInputRef}
          type="file"
          onChange={handleFileUpload}
          className="hidden"
        />
        <button
          onClick={() => fileInputRef.current?.click()}
          disabled={loading}
          className={`upload-button ${
            isDarkMode 
              ? 'upload-button-dark' 
              : 'upload-button-light'
          }`}
        >
          <Upload size={18} />
          Upload File
        </button>
      </div>

      {loading && (
        <div className="loading-center">
          <div className="loading-spinner-small"></div>
        </div>
      )}

      <div className={`table-container ${isDarkMode ? 'table-container-dark' : 'table-container-light'}`}>
        <table className="data-table">
          <thead className={isDarkMode ? 'table-header-dark' : 'table-header-light'}>
            <tr>
              <th className="table-header-cell">Name</th>
              <th className="table-header-cell">Size</th>
              <th className="table-header-cell">Uploaded</th>
              <th className="table-header-cell">Actions</th>
            </tr>
          </thead>
          <tbody className={`table-body ${isDarkMode ? 'table-body-dark' : 'table-body-light'}`}>
            {files.map((file) => (
              <tr key={file.id} className={isDarkMode ? 'table-row-dark' : 'table-row-light'}>
                <td className={`table-cell ${isDarkMode ? 'text-white' : 'text-gray-900'}`}>
                  {file.original_name}
                </td>
                <td className={`table-cell ${isDarkMode ? 'text-gray-300' : 'text-gray-500'}`}>
                  {formatFileSize(file.size)}
                </td>
                <td className={`table-cell ${isDarkMode ? 'text-gray-300' : 'text-gray-500'}`}>
                  {formatDate(file.uploaded_at)}
                </td>
                <td className="table-cell relative" ref={dropdownRef}>
                  <button
                    onClick={() => setOpenDropdown(openDropdown === file.id ? null : file.id)}
                    className={`actions-button ${
                      isDarkMode ? 'actions-button-dark' : 'actions-button-light'
                    }`}
                  >
                    Actions
                    <ChevronDown size={16} />
                  </button>
                  
                  {openDropdown === file.id && (
                    <div className={`dropdown-menu ${
                      isDarkMode ? 'dropdown-menu-dark' : 'dropdown-menu-light'
                    }`}>
                      <button
                        onClick={() => handleFileAction('share', file.id)}
                        className={`dropdown-item ${
                          isDarkMode ? 'dropdown-item-dark' : 'dropdown-item-light'
                        }`}
                      >
                        <Share size={16} />
                        Share
                      </button>
                      <button
                        onClick={() => handleFileAction('delete', file.id)}
                        className="dropdown-item dropdown-item-danger"
                      >
                        <Trash2 size={16} />
                        Delete
                      </button>
                    </div>
                  )}
                </td>
              </tr>
            ))}
          </tbody>
        </table>
        
        {files.length === 0 && !loading && (
          <div className={`empty-state ${isDarkMode ? 'text-gray-400' : 'text-gray-500'}`}>
            No files uploaded yet
          </div>
        )}
      </div>
    </div>
  );
};